// FluidDataSet is a collection of multi-dimension points. Each point is a snapshot of the values of each dimension.
//
// But what happens when these values change in time? So far, we have either sampled-and-held, or we have used statistical tools (fluid.bufstats) to boil down and model a profile, but we have lost the actual gesture in time.
//
// Enters FluidDataSeries. It allows to associate with an identifier a series of points. When a point is in a series is called a frame, as it has an order. Every frame in every series need to be of the same number of dimensions... but each series can be of a different length for frames. A short gesture might have 5 time points, or frames, and a longer one in the same FluidDataSeries object might have 15 frames.
//
// The interface with SC is mostly done via with Buffers, like for FluidDataSets.
// To add a series, one can add the whole series as a multidimensional, multichannel buffer. Contrary to FluidDataSet, each channel is a dimension, and time is what is normally the case in a buffer (frame-wise).

// Example 1
"Ex-1-1.scd".resolveRelative.openOS

// This buffer dimension arrangement is compatible with the FluidBuf* objects.
"Ex-1-2.scd".resolveRelative.openOS

// So far, we covered addSeries, getSeries, print, dump, clear, kNearest

// We can also add one frame at a time into series. This can be helpful for realtime entry
"Ex-1-1-redone-by-frame.scd".resolveRelative.openOS

// It is also possible to retrieve a time slice as dataset. The first slice will always have all the series present, and the further we go in time the smaller the output dataset will be, since some series will have ended.
"Ex-1-3-retrieve-dataset.scd".resolveRelative.openOS

// Like FluidDataSets, one can
// load / dump to native dictionaries
// read / write to a json file
// merge 2 FluidDataSeries, potentially overwriting shared identifiers