//This is an homage to the classic example by Fiebrink et al

//It uses dynamic time warping (DTW) to find the most likely 'class' of an input by finding its K nearest items's class and assuming it is most likely the same. It is similar to the knnclassifier but instead of taking points to points via a kdtree, it compares the various time series using DTW.

//1. identify a first gesture class
~gestureClass = \gesture_1;

//2. draw 5 instances of that gesture in the GUI - do not close the window after
(
~xypoint = Buffer.alloc(s,2);
~entry = FluidDataSet(s);
~gestureXY = Buffer(s);
~count = 0;
~series = FluidDataSeries(s);
~labels = FluidLabelSet(s);
~dtwc = FluidDTWClassifier(s);
~makingEntries = true;

SynthDef(\writer, {var trig = Impulse.kr(50); FluidDataSetWr.kr(~entry, buf: ~xypoint, idNumber: PulseCount.kr(trig),trig:trig)}).send;

FluidPlotter(bounds: Rect(top: Window.screenBounds.height-400, width: 400,height: 400), mouseMoveAction: {|...in|
	if (in[5].notNil, {
		//clear the ds
		~entry.clear;
		in[0].dict = Dictionary.newFrom(["cols",2,"data",Dictionary.newFrom([])]);
		//write the value to buf
		~xypoint.setn(0,in[[1,2]]);
		//start the writer function that has a constant frequency
		~timedWriter = Synth(\writer);
	}, {
		if (in[4].notNil, {
			//stop the writer function
			~timedWriter.free;
			//transfer to buf
			~entry.toBuffer(~gestureXY,action: {
				if (~makingEntries, {
					//make an entry in the FluidDataSeries with a given identifier
					~series.addSeries(~count.asSymbol, ~gestureXY);
					//and make an entry in the FluidLabelSet with the same identifier
					~labels.addLabel(~count.asSymbol, ~gestureClass);
				},{
					~dtwc.predictSeries(~gestureXY,{|x|x.postln});
				});

			});
		}, {
			~xypoint.setn(0,in[[1,2]]);
			in[0].addPoint_(~count.asSymbol,in[1],in[2]);
			~count = ~count + 1;
		});
	});
})
)

//3. identify a second gesture class
~gestureClass = \gesture_2;

//4. draw 5 instances of that gesture in the GUI

//5. print to see
~series.print
~labels.print

//6. switch to query
~makingEntries = false;

//7. fit the series to their respective label
~dtwc.fit(~series,~labels);

//8. draw a gesture in the GUI and see in the post window which class it identifies