// Slice and analyse, code adapted from the end of the 2d plotter video tutorial series

~src = Buffer.read(s, FluidFilesPath("Nicol-LoopE-M.wav"));

// slice the buffer in non real-time
(
~indices = Buffer(s);
FluidBufOnsetSlice.processBlocking(s,~src,metric:9,threshold:0.5,indices:~indices,action:{
	"found % slice points".format(~indices.numFrames).postln;
	"average duration per slice: %".format(~src.duration / (~indices.numFrames+1)).postln;
});
)

// analysis in 13 dimensions
(
~analyses = FluidDataSeries(s);
~indices.loadToFloatArray(action:{
	arg fa;
	var mfccs = Buffer(s);

	fa.doAdjacentPairs{
		arg start, end, i;
		var num = end - start;

		FluidBufMFCC.processBlocking(s,~src,start,num,features:mfccs,numCoeffs:13,startCoeff:1);

		~analyses.addSeries(i,mfccs);

		"analyzing slice % / %".format(i+1,fa.size-1).postln;

		if((i%100) == 99){s.sync;}
	};

	s.sync;

	~analyses.print;
});
)

// pick a slice to query
~query = Buffer()
~analyses.getSeries(\0,~query,{\done.postln})
~query.plot

// a function to play a slice
(
~play_slice = {
	arg index;
	{
		var startsamp = Index.kr(~indices,index);
		var stopsamp = Index.kr(~indices,index+1);
		var phs = Phasor.ar(0,BufRateScale.ir(~src),startsamp,stopsamp);
		var sig = BufRd.ar(1,~src,phs);
		var dursecs = (stopsamp - startsamp) / BufSampleRate.ir(~src);
		var env;

		dursecs = min(dursecs,1);

		env = EnvGen.kr(Env([0,1,1,0],[0.03,dursecs-0.06,0.03]),doneAction:2);
		sig.dup * env;
	}.play;
};
)

// play the slice
~play_slice.(0)

// query for 2 nearest neighbours
~analyses.kNearest(~query,2,{|x|~nearest = x.postln})

// indeed, as we are querying from the same pool, the nearest will be itself, so we take the 2nd nearest :)
~play_slice.(~nearest[1].asInteger)

// you can load a full folder of sound but be careful, this is bruteforce search so large dataseries will be long to query

// the folder containing the sounds
~folder = FluidFilesPath();

// load into a buffer
~loader = FluidLoadFolder(~folder).play(s,{"done loading folder".postln;});

// make mono
(
if(~loader.buffer.numChannels > 1){
	~src = Buffer(s);
	~loader.buffer.numChannels.do{
		arg chan_i;
		FluidBufCompose.processBlocking(s,
			~loader.buffer,
			startChan:chan_i,
			numChans:1,
			gain:~loader.buffer.numChannels.reciprocal,
			destination:~src,
			destGain:1,
			action:{"copied channel: %".format(chan_i).postln}
		);
	};
}{
	"loader buffer is already mono".postln;
	~src = ~loader.buffer;
};
)